@@ -1,14 +1,14 @@
 ;;; org-journal.el --- a simple org-mode based journaling mode -*- lexical-binding: t; -*-
 
-;; Copyright 2013-2022 Bastian Bechtold
+;; Copyright 2013-2024 Bastian Bechtold
 
 ;; Author: Bastian Bechtold
 ;;         Christian Schwarzgruber
 
 ;; URL: http://github.com/bastibe/org-journal
-;; Package-Version: 2.2.0
-;; Package-Revision: 18df4d5ae5e1
-;; Package-Requires: ((emacs "25.1") (org "9.1"))
+;; Package-Version: 20250525.951
+;; Package-Revision: 8b9b46f988ed
+;; Package-Requires: ((emacs "26.1") (org "9.1"))
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions are met:
@@ -104,58 +104,51 @@
 ;; Silent byte-compiler
 (defvar view-exit-action)
 (declare-function org-collect-keywords "org")
+(defvar age-armor)
+(defvar age-file-encrypt-to)
+(declare-function age-make-context "ext:age.el")
+(declare-function age-select-keys "ext:age.el")
 
 (when (version< org-version "9.2")
   (defalias 'org-set-tags-to 'org-set-tags))
 
-(unless (fboundp 'org--tag-add-to-alist)
-  ;; This function can be removed once emacs-26 es required or de-facto standard.
-  (defun org-tag-add-to-alist (alist1 alist2)
-    "Append ALIST1 elements to ALIST2 if they are not there yet.
+(eval-when-compile
+  (unless (fboundp 'org--tag-add-to-alist)
+    ;; This function can be removed once emacs-26 is required or de-facto standard.
+    (defun org-tag-add-to-alist (alist1 alist2)
+      "Append ALIST1 elements to ALIST2 if they are not there yet.
 
 From branch \"emacs-26\", added for compatibility.
 "
-    (cond
-      ((null alist2) alist1)
-      ((null alist1) alist2)
-      (t (let ((alist2-cars (mapcar (lambda (x) (car-safe x)) alist2))
-               to-add)
-           (dolist (i alist1)
-             (unless (member (car-safe i) alist2-cars)
-               (push i to-add)))
-           (append to-add alist2)))))
-  (defalias 'org--tag-add-to-alist 'org-tag-add-to-alist))
+      (cond
+       ((null alist2) alist1)
+       ((null alist1) alist2)
+       (t (let ((alist2-cars (mapcar (lambda (x) (car-safe x)) alist2))
+                to-add)
+            (dolist (i alist1)
+              (unless (member (car-safe i) alist2-cars)
+                (push i to-add)))
+            (append to-add alist2)))))
+    (defalias 'org--tag-add-to-alist 'org-tag-add-to-alist)))
 
+
 ;;; Customizable variables
 (defgroup org-journal nil
   "Settings for the personal journal"
   :group 'org
   :group 'org-journal)
 
-(defface org-journal-highlight
-    '((t (:foreground "#ff1493")))
-  "Face for highlighting org-journal buffers.")
-
-(defun org-journal-highlight (str)
-  "Highlight STR in current-buffer"
-  (goto-char (point-min))
-  (while (search-forward str nil t)
-    (put-text-property (match-beginning 0) (match-end 0) 'font-lock-face 'org-journal-highlight)))
-
-(defface org-journal-calendar-entry-face
-    '((t (:foreground "#aa0000" :slant italic)))
-  "Face for highlighting org-journal entries in M-x calendar.")
-
-(defface org-journal-calendar-scheduled-face
-    '((t (:foreground "#600000" :slant italic)))
-  "Face for highlighting future org-journal entries in M-x calendar.")
+(defcustom org-journal-mode-hook '(turn-on-visual-line-mode
+                                   org-journal-default-enable-encryption)
+  "Hook to run when `org-journal-mode' is loaded."
+  :type 'hook)
 
 (defcustom org-journal-file-type 'daily
   "What type of journal file to create.
 
-When switching from daily to weekly, monthly, yearly, or from weekly,
-monthly, yearly to daily, you need to invalidate the cache. This has currently
-to be done manually by calling `org-journal-invalidate-cache'."
+When switching from `daily' to `weekly', `monthly', `yearly', or from `weekly',
+`monthly', `yearly' to `daily', you need to invalidate the cache. This has
+currently to be done manually by calling `org-journal-invalidate-cache'."
   :type '(choice
           (const :tag "Daily" daily)
           (const :tag "Weekly" weekly)
@@ -163,7 +156,7 @@ to be done manually by calling `org-journal-invalidate-cache'."
           (const :tag "Yearly" yearly)))
 
 (defcustom org-journal-start-on-weekday 1
-  "When `org-journal-file-type' is set to 'weekly, start the week on this day.
+  "When `org-journal-file-type' is set to `weekly', start the week on this day.
 
 1 for Monday, ..., and 7 for Sunday."
   :type '(choice
@@ -258,12 +251,29 @@ passphrase to encrypt/decrypt it."
   :type 'boolean)
 
 (defcustom org-journal-encrypt-journal nil
-  "If non-nil, encrypt journal files using gpg.
+  "If non-nil, encrypt journal files.
+org-journal doesn't do the encryption itself,
+so you must have set up a transparent way of
+decryption and encryption.
+By default this is done using epa (EasyPG),
+which will automatically
 
-The journal files will have the file extension \".gpg\"."
+By default the journal files will have the file extension \".gpg\".
+You can change the file extension and therefore the encryption used via
+the `org-journal-encryption-format' variable."
   :type 'boolean)
 
-(defcustom org-journal-encrypt-on 'before-save-hook
+(defcustom org-journal-encryption-extension "gpg"
+  "The type of encryption to be used by apending the extension to
+the journal file name.
+Encryption is only used when `org-journal-encrypt-journal' is set.
+
+Currently supported extensions are \".gpg\" via EasyPG (epa) and
+\".age\" via age.el."
+  :type 'string)
+
+(define-obsolete-variable-alias  'org-journal-encrypt-on 'org-journal-encrypt-on-hook-fn "2.3.0")
+(defcustom org-journal-encrypt-on-hook-fn 'before-save-hook
   "Hook on which to encrypt entries.
 
 It can be set to other hooks like `kill-buffer-hook'."
@@ -273,7 +283,8 @@ It can be set to other hooks like `kill-buffer-hook'."
   "Add current and future org-journal files to `org-agenda-files' when non-nil."
   :type 'boolean)
 
-(defcustom org-journal-find-file 'find-file-other-window
+(define-obsolete-variable-alias  'org-journal-find-file 'org-journal-find-file-fn "2.3.0")
+(defcustom org-journal-find-file-fn 'find-file-other-window
   "The function to use when opening an entry.
 
 Set this to `find-file' if you don't want org-journal to split your window."
@@ -285,15 +296,16 @@ Set this to `find-file' if you don't want org-journal to split your window."
   See agenda tags view match description for the format of this."
   :type 'string)
 
-(defcustom org-journal-skip-carryover-drawers nil
+(defcustom org-journal-skip-carryover-drawers '()
   "By default, we carry over all the drawers associated with the items.
 
 This option can be used to skip certain drawers being carried over.
 The drawers listed here will be wiped completely, when the item gets carried
 over."
-  :type 'list)
+  :type '(repeat string))
 
-(defcustom org-journal-handle-old-carryover 'org-journal-delete-old-carryover
+(define-obsolete-variable-alias 'org-journal-handle-old-carryover 'org-journal-handle-old-carryover-fn "2.3.0")
+(defcustom org-journal-handle-old-carryover-fn 'org-journal-delete-old-carryover
   "The function to handle the carryover entries in the previous journal.
 
 This function takes one argument, which is a list of the carryover entries
@@ -395,6 +407,24 @@ This prefix key is used for:
 - `org-journal-search' (key \"s\")"
   :type 'string)
 
+(defcustom org-journal-scheduled-string ""      ; or org-scheduled-string
+  "String added before a time stamp for schedules."
+  :type 'string)
+
+
+(defface org-journal-highlight
+    '((t (:foreground "#ff1493")))
+  "Face for highlighting org-journal buffers.")
+
+(defface org-journal-calendar-entry-face
+    '((t (:foreground "#aa0000" :slant italic)))
+  "Face for highlighting org-journal entries in M-x calendar.")
+
+(defface org-journal-calendar-scheduled-face
+    '((t (:foreground "#600000" :slant italic)))
+  "Face for highlighting future org-journal entries in M-x calendar.")
+
+
 (defvar org-journal-after-entry-create-hook nil
   "Hook called after journal entry creation.")
 
@@ -405,6 +435,37 @@ This runs once per date, before `org-journal-after-entry-create-hook'.")
 
 (defvar org-journal--search-buffer "*Org-journal search*")
 
+(defvar org-journal-search-history nil)
+
+(defvar org-journal--sorted-dates nil)
+
+(defvar org-journal--kill-buffer nil
+  "Will be set to the `t' if `org-journal--open-entry' is visiting a
+buffer not open already, otherwise `nil'.")
+
+(defvar org-journal--format-rx-alist
+  '(("%[aAbB]" . "\\\\(?4:[[:alpha:]]\\\\{3,\\\\}\\\\)")
+    ("%d" . "\\\\(?3:[0-9]\\\\{2\\\\}\\\\)")
+    ("%m" . "\\\\(?2:[0-9]\\\\{2\\\\}\\\\)")
+    ("%Y" . "\\\\(?1:[0-9]\\\\{4\\\\}\\\\)")
+    ("%V" . "[0-9]\\\\{2\\\\}")))
+
+(defvar org-journal--created-re "^ *:CREATED: +.*$"  "Regex to find created property.")
+
+(defvar org-time-was-given)
+
+(defvar org-end-time-was-given)
+
+(defconst org-journal--cache-file
+  (expand-file-name "org-journal.cache" user-emacs-directory)
+  "Cache file for `org-journal--dates'.")
+
+(defvar org-journal--dates (make-hash-table :test 'equal)
+  "Hash table for journal dates.
+
+The key is a journal date entry, and the value of the key is of the form
+\(FILENAME \(FILE MODIFICATION TIME\)\).")
+
 
 ;;;###autoload
 (add-hook 'calendar-today-visible-hook 'org-journal-mark-entries)
@@ -421,11 +482,6 @@ This runs once per date, before `org-journal-after-entry-create-hook'.")
     (org-journal--set-current-tag-alist))
   (run-mode-hooks))
 
-;;;###autoload
-(define-obsolete-function-alias 'org-journal-open-next-entry 'org-journal-next-entry "2.1.0")
-;;;###autoload
-(define-obsolete-function-alias 'org-journal-open-previous-entry 'org-journal-previous-entry "2.1.0")
-
 ;; Key bindings
 (when (and (stringp org-journal-prefix-key) (not (string-empty-p org-journal-prefix-key)))
   (let ((command-table '(("f" . org-journal-next-entry)
@@ -468,12 +524,21 @@ before it will be deposed."
      (with-current-buffer buf
        (unless buffer-exists
          (insert-file-contents ,file))
-       (setq result (progn ,@body)))
+       ;; Use `let' to change the local value of `major-mode' and avoid
+       ;; activating `org-mode' directly and incurring a performance penalty.
+       (setq result (let ((major-mode 'org-mode))
+                      (progn ,@body))))
      (unless buffer-exists
        (kill-buffer buf))
      result))
 (def-edebug-spec org-journal--with-journal (form body))
 
+(defun org-journal-highlight (str)
+  "Highlight STR in current-buffer"
+  (goto-char (point-min))
+  (while (search-forward str nil t)
+    (put-text-property (match-beginning 0) (match-end 0) 'font-lock-face 'org-journal-highlight)))
+
 (defun org-journal-after-save-hook ()
   "Update agenda files and dates."
   (org-journal--update-org-agenda-files)
@@ -492,14 +557,7 @@ before it will be deposed."
   "Return the current journal file pattern"
   (concat (file-name-as-directory (file-truename org-journal-dir))
           (org-journal--format->regex org-journal-file-format)
-          "\\(\\.gpg\\)?\\'"))
-
-(defvar org-journal--format-rx-alist
-  '(("%[aAbB]" . "\\\\(?4:[a-zA-Z]\\\\{3,\\\\}\\\\)")
-    ("%d" . "\\\\(?3:[0-9]\\\\{2\\\\}\\\\)")
-    ("%m" . "\\\\(?2:[0-9]\\\\{2\\\\}\\\\)")
-    ("%Y" . "\\\\(?1:[0-9]\\\\{4\\\\}\\\\)")
-    ("%V" . "[0-9]\\\\{2\\\\}")))
+          "\\(\.\\(gpg\\|age\\)\\)?\\'"))
 
 (defun org-journal--format->regex (format)
   (cl-loop
@@ -509,8 +567,6 @@ before it will be deposed."
     ;; Ignore extra timestamp elements rather than erroring out
     finally return (replace-regexp-in-string "[]>]$" "[A-z0-9: ]*\\&" format)))
 
-(defvar org-journal--created-re "^ *:CREATED: +.*$"  "Regex to find created property.")
-
 (defun org-journal--search-forward-created (date &optional bound noerror count)
   "Search for CREATED tag with date."
   (re-search-forward
@@ -557,13 +613,13 @@ Convert OLD-FORMAT or input to `org-journal-created-property-timestamp-format'."
 (defun org-journal--convert-time-to-file-type-time (&optional time)
   "Converts TIME to the file type format date.
 
-If `org-journal-file-type' is 'weekly, the TIME will be rounded to
+If `org-journal-file-type' is `weekly', the TIME will be rounded to
 the first date of the week.
 
-If `org-journal-file-type' is 'monthly, the TIME will be rounded to
+If `org-journal-file-type' is `monthly', the TIME will be rounded to
 the first date of the month.
 
-If `org-journal-file-type' is 'yearly, the TIME will be rounded to
+If `org-journal-file-type' is `yearly', the TIME will be rounded to
 the first date of the year."
   (or time (setq time (current-time)))
   (pcase org-journal-file-type
@@ -606,7 +662,7 @@ If no TIME is given, uses the current time."
                                     (org-journal--convert-time-to-file-type-time time))
                 org-journal-dir))))
     (when (and org-journal-encrypt-journal (not (file-exists-p file)))
-      (setq file (concat file ".gpg")))
+      (setq file (concat file "." org-journal-encryption-extension)))
     file))
 
 (defun org-journal--create-journal-dir ()
@@ -684,37 +740,37 @@ This allows the use of `org-journal-tag-alist' and
     (unless (if (org-journal--daily-p)
                 (or (search-forward entry-header nil t) (and (goto-char (point-max)) nil))
               (cl-loop
-                with date = (decode-time time)
-                with file-dates = (sort (org-journal--file->calendar-dates (buffer-file-name))
-                                        (lambda (a b)
-                                          (org-journal--calendar-date-compare b a)))
-                with entry
-                initially (setq date (list (nth 4 date) (nth 3 date) (nth 5 date)))
-                unless file-dates ;; New entry at bof
-                do
-                  (unless (re-search-forward (concat "^\\(" org-outline-regexp "\\)") nil t)
-                    (goto-char (point-max)))
-                  (if (org-at-heading-p)
-                      (progn
-                        (beginning-of-line)
-                        (insert "\n")
-                        (forward-line -1))
-                    (forward-line -1)
-                    (end-of-line))
-                and return nil
-                while file-dates
-                do
-                  (setq entry (car file-dates)
-                        file-dates (cdr file-dates))
-                if (or (org-journal--calendar-date-compare entry date) (equal entry date))
-                do
-                  (org-journal--search-forward-created entry)
-                  (when (org-journal--calendar-date-compare entry date) ;; New entry at eof, or somewhere in-between
-                    (org-end-of-subtree))
-                and return (equal entry date))) ;; If an entry exists don't create a header
+                    with date = (decode-time time)
+                    with file-dates = (sort (org-journal--file->calendar-dates (buffer-file-name))
+                                            (lambda (a b)
+                                              (org-journal--calendar-date-compare b a)))
+                    with entry
+                    initially (setq date (list (nth 4 date) (nth 3 date) (nth 5 date)))
+                    unless file-dates ;; New entry at bof
+                    do
+                    (unless (re-search-forward (concat "^\\(" org-outline-regexp "\\)") nil t)
+                      (goto-char (point-max)))
+                    (if (org-at-heading-p)
+                        (progn
+                          (beginning-of-line)
+                          (insert "\n")
+                          (forward-line -1))
+                      (forward-line -1)
+                      (end-of-line))
+                    and return nil
+                    while file-dates
+                    do
+                    (setq entry (car file-dates)
+                          file-dates (cdr file-dates))
+                    if (or (org-journal--calendar-date-compare entry date) (equal entry date))
+                    do
+                    (org-journal--search-forward-created entry)
+                    (when (org-journal--calendar-date-compare entry date) ;; New entry at eof, or somewhere in-between
+                      (org-end-of-subtree))
+                    and return (equal entry date))) ;; If an entry exists don't create a header
 
 
-      (when (looking-back "[^\t ]" (point-at-bol))
+      (when (looking-back "[^\t ]" (line-beginning-position))
         (insert "\n"))
       (insert entry-header)
 
@@ -729,7 +785,7 @@ This allows the use of `org-journal-tag-alist' and
           (org-set-tags org-crypt-tag-matcher)))
       (run-hooks 'org-journal-after-header-create-hook))))
 
-(defun org-journal--insert-entry (time org-extend-today-until-active-p)
+(defun org-journal--insert-entry (time org-extend-today-until-active-p &optional no-timestamp)
   "Insert a new entry."
   (unless (eq (current-column) 0) (insert "\n"))
   (let* ((day-discrepancy (- (time-to-days (current-time)) (time-to-days time)))
@@ -745,11 +801,11 @@ This allows the use of `org-journal-tag-alist' and
                          (format-time-string org-journal-time-format)))
                       ;; “time” is on some other day, use blank timestamp
                       (t ""))))
-    (insert org-journal-time-prefix timestamp))
+    (insert org-journal-time-prefix (if no-timestamp "" timestamp)))
   (run-hooks 'org-journal-after-entry-create-hook))
 
 ;;;###autoload
-(defun org-journal-new-entry (prefix &optional time)
+(defun org-journal-new-entry (prefix &optional time no-timestamp)
   "Open today's journal file and start a new entry.
 
 With a PREFIX arg, open the today's file, create a heading if it
@@ -803,7 +859,7 @@ hook is run."
       (goto-char (point-max)))
 
     (when should-add-entry-p
-      (org-journal--insert-entry time org-extend-today-until-active-p))
+      (org-journal--insert-entry time org-extend-today-until-active-p no-timestamp))
 
     (if (and org-journal-hide-entries-p (org-journal--time-entry-level))
         (outline-hide-sublevels (org-journal--time-entry-level))
@@ -812,10 +868,6 @@ hook is run."
     (when should-add-entry-p
       (outline-show-entry))))
 
-(defvar org-journal--kill-buffer nil
-  "Will be set to the `t' if `org-journal--open-entry' is visiting a
-buffer not open already, otherwise `nil'.")
-
 (defun org-journal--empty-journal-p (prev-buffer)
   (let (entry)
     (with-current-buffer prev-buffer (save-buffer))
@@ -919,8 +971,8 @@ to process the carryover entries in `prev-buffer'."
       (save-excursion
         (while (re-search-forward "<\\([0-9]\\{4\\}-[0-9]\\{2\\}-[0-9]\\{2\\}\\( [a-z]+\\)?\\)>" nil t)
           (unless (save-excursion
-                    (goto-char (point-at-bol))
-                    (re-search-forward "\\<\\(SCHEDULED\\|DEADLINE\\):" (point-at-eol) t))
+                    (goto-char (line-beginning-position))
+                    (re-search-forward "\\<\\(SCHEDULED\\|DEADLINE\\):" (line-end-position) t))
             (replace-match
              (format-time-string "%Y-%m-%d %a"
                                  (org-journal--calendar-date->time
@@ -1002,7 +1054,7 @@ previous day's file to the current file."
               end (save-excursion (outline-next-heading) (point))
               text (buffer-substring-no-properties start end))
         (push (cons start (cons end text)) carryover-item-with-parents)))
-    (setq start (point-at-bol)
+    (setq start (line-beginning-position)
           end (progn (outline-end-of-subtree) (outline-next-heading) (point))
           text (buffer-substring-no-properties start end))
     (setq carryover-item-with-parents (append carryover-item-with-parents (list (cons start (cons end text)))))))
@@ -1052,7 +1104,7 @@ Month and Day capture group default to 1."
   "Return journal calendar-date from current buffer.
 
 This is the counterpart of `org-journal--file-name->calendar-date' for
-'weekly, 'monthly and 'yearly journal files."
+`weekly', `monthly' and `yearly' journal files."
   (let ((re (org-journal--format->regex org-journal-created-property-timestamp-format))
         date)
     (setq date (org-entry-get (point) "CREATED"))
@@ -1107,40 +1159,68 @@ insert just the heading."
 
 With non-nil prefix argument create a regular entry instead of a TODO entry."
   (interactive "P")
-  (let ((time (or scheduled-time (org-time-string-to-time (org-read-date nil nil nil "Date:"))))
-        org-journal-carryover-items)
+  (let* ((org-time-was-given nil) (org-end-time-was-given nil)
+         (time (or scheduled-time (org-time-string-to-time (org-read-date nil nil nil "Date:"))))
+         org-journal-carryover-items)
     (when (time-less-p time (current-time))
       (user-error "Scheduled time needs to be in the future"))
-    (org-journal-new-entry nil time)
+    (org-journal-new-entry nil time t)
     (unless prefix
       (insert "TODO "))
+    (if org-time-was-given
+        (insert (format-time-string org-journal-time-format time)))
     (save-excursion
-      (insert "\n")
-      (org-insert-time-stamp time t))))
+      (insert "\n"
+              org-journal-scheduled-string
+              (if (> (length org-journal-scheduled-string) 0) " " ""))
+      (org-insert-time-stamp
+       time org-time-was-given nil nil nil (list org-end-time-was-given)))))
 
 ;;;###autoload
 (defun org-journal-reschedule-scheduled-entry (&optional time)
   "Reschedule an entry in the future."
   (interactive "P")
-  (or time (setq time (org-time-string-to-time (org-read-date nil nil nil "Data:"))))
-  (when (time-less-p time (current-time))
-    (user-error "Scheduled time needs to be in the future"))
-  (save-excursion
-    (save-restriction
-      (org-back-to-heading)
-      (org-narrow-to-subtree)
-      (if (re-search-forward org-ts-regexp (line-end-position 2) t)
-          (replace-match "")
-        (org-end-of-subtree)
-        (insert "\n"))
-      (org-insert-time-stamp time)
-      (org-cut-subtree))
-    (let (org-journal-carryover-items)
-      (org-save-outline-visibility t
-        (org-journal-new-entry t time)
-        (when (looking-back "[^\t ]" (point-at-bol) t)
+  (let ((org-time-was-given nil) (org-end-time-was-given nil))
+    (or time (setq time (org-time-string-to-time (org-read-date nil nil nil "Date:"))))
+    (when (time-less-p time (current-time))
+      (user-error "Scheduled time needs to be in the future"))
+    (save-excursion
+      (save-restriction
+        (org-back-to-heading)
+        (org-narrow-to-subtree)
+        ;; update time after org-journal-time-prefix
+        (let ((regexp
+               (concat
+                "^"
+                (regexp-quote org-journal-time-prefix)
+                "\\(TODO \\)?"
+                "\\("
+                (replace-regexp-in-string "[[:digit:]]" "[[:digit:]]"
+                                          (format-time-string org-journal-time-format '(0 0) t))
+                "\\)")))
+          (if (re-search-forward regexp (line-end-position) t)
+              (progn
+                (delete-region (match-beginning 2) (match-end 2))
+                (if org-time-was-given
+                    (insert (format-time-string org-journal-time-format time))))))
+        ;; update time of timestamp (<...>)
+        (org-back-to-heading)
+        (if (re-search-forward (concat (regexp-quote org-journal-scheduled-string)
+                                       "[[:blank:]]*" org-ts-regexp)
+                               nil t)
+            (replace-match "")
+          (org-end-of-subtree)
           (insert "\n"))
-        (org-yank)))))
+        (insert org-journal-scheduled-string
+                (if (> (length org-journal-scheduled-string) 0) " " ""))
+        (org-insert-time-stamp time org-time-was-given nil nil nil (list org-end-time-was-given))
+        (org-cut-subtree))
+      (let (org-journal-carryover-items)
+        (org-save-outline-visibility t
+                                     (org-journal-new-entry t time)
+                                     (when (looking-back "[^\t ]" (line-beginning-position) t)
+                                       (insert "\n"))
+                                     (org-yank))))))
 
 (defun org-journal--goto-entry (date)
   "Goto DATE entry in current journal file."
@@ -1208,7 +1288,9 @@ If NO-SELECT is non-nil, open it, but don't show it."
 (defun org-journal-open-current-journal-file ()
   "Open the current journal file"
   (interactive)
-  (let ((org-journal-file (org-journal--get-entry-path)))
+  (let ((org-journal-file (org-journal--get-entry-path
+                           (time-subtract (current-time)
+                                          (* 3600 org-extend-today-until)))))
     (if (file-exists-p org-journal-file)
         (progn
           (funcall org-journal-find-file org-journal-file)
@@ -1231,19 +1313,10 @@ If NO-SELECT is non-nil, open it, but don't show it."
         (predicate (lambda (file-path)
                      (and (string-match-p (org-journal--dir-and-file-format->pattern) file-path)
                           (or org-journal-encrypt-journal
-                              (not (string-match-p "\.gpg$" file-path)))))))
+                              (not (string-match-p "\.\\(gpg\\|age\\)$" file-path)))))))
     (seq-filter predicate file-list)))
 
-(defconst org-journal--cache-file
-  (expand-file-name "org-journal.cache" user-emacs-directory)
-  "Cache file for `org-journal--dates'.")
-
-(defvar org-journal--dates (make-hash-table :test 'equal)
-  "Hash table for journal dates.
-
-The key is a journal date entry, and the value of the key is of the form
-\(FILENAME \(FILE MODIFICATION TIME\)\).")
-
+
 ;;;###autoload
 (defun org-journal-invalidate-cache ()
   "Clear `org-journal--dates' hash table, and the cache file."
@@ -1287,11 +1360,9 @@ The key is a journal date entry, and the value of the key is of the form
       (when (file-exists-p org-journal--cache-file)
         (with-temp-buffer
           (insert-file-contents org-journal--cache-file)
-          (setq org-journal--dates (read (buffer-substring (point-at-bol) (point-at-eol))))))))
+          (setq org-journal--dates (read (buffer-substring (line-beginning-position) (line-end-position))))))))
   (org-journal--sort-dates))
 
-(defvar org-journal--sorted-dates nil)
-
 (defun org-journal--sort-dates ()
   "Flatten and sort dates, and assign the result to `org-journal--sorted-dates'."
   (setq org-journal--sorted-dates (sort (hash-table-keys org-journal--dates) 'org-journal--calendar-date-compare)))
@@ -1379,7 +1450,7 @@ from oldest to newest."
       (progn
         (org-up-heading-safe)
         (org-back-to-heading)
-        (outline-hide-other)
+        (if org-journal-hide-entries-p (outline-hide-other))
         (outline-show-subtree))
     (outline-show-all)))
 
@@ -1402,6 +1473,7 @@ is nil or avoid switching when NOSELECT is non-nil."
                      (setq point (org-journal--search-forward-created
                                   (list (nth 4 time) (nth 3 time) (nth 5 time))
                                   nil t))))))
+
         (progn
           ;; Use `find-file-noselect' instead of `view-file' as it does not respect `auto-mode-alist'
           (with-current-buffer buf
@@ -1465,6 +1537,7 @@ If prev is non-nil open previous entry instead of next."
   (interactive)
   (org-journal--next-entry t))
 
+
 ;;; Journal search facilities
 
 ;;;###autoload
@@ -1484,8 +1557,6 @@ If a prefix argument is given, search all dates."
     (setcar (cdr end) (1+ (cadr end)))
     (org-journal--search-by-string str start end)))
 
-(defvar org-journal-search-history nil)
-
 ;;;###autoload
 (defun org-journal-search-calendar-week (str)
   "Search for a string within a current calendar-mode week entries."
@@ -1642,8 +1713,8 @@ Think of this as a faster, less fancy version of your `org-agenda'."
   "Return read period.
 
 If the PERIOD-NAME is nil, then ask the user for period start/end.
-If PERIOD-NAME is 'forever, set the period from the beginning of time
-to eternity. If PERIOD-NAME is a symbol equal to 'week, 'month or 'year
+If PERIOD-NAME is `forever', set the period from the beginning of time
+to eternity. If PERIOD-NAME is a symbol equal to `week', `month' or `year'
 then use current week, month or year from the calendar, accordingly."
   (cond
     ;; no period-name? ask the user for input
@@ -1822,7 +1893,7 @@ If STR is empty, search for all entries using `org-journal-time-prefix'."
 (unless org-journal-search-mode-map
   (setq org-journal-search-mode-map
         (let ((map (make-sparse-keymap)))
-          (define-key map "q" 'kill-this-buffer)
+          (define-key map "q" 'kill-current-buffer)
           (define-key map (kbd "<tab>") 'org-journal--search-next)
           (define-key map (kbd "<backtab>") 'org-journal--search-prev)
           (define-key map "n" 'org-journal--search-next)
@@ -1873,9 +1944,18 @@ If STR is empty, search for all entries using `org-journal-time-prefix'."
 
 (defun org-journal-re-encrypt-journals (recipient)
   "Re-encrypt journal files."
-  (interactive (list (epa-select-keys (epg-make-context epa-protocol)
-                                      "Select new recipient for encryption.
-Only one recipient is supported.  ")))
+  (interactive (list (cond
+                       ((string= org-journal-encryption-extension "gpg")
+                        (epa-select-keys (epg-make-context epa-protocol)
+                                         "Select new recipient for encryption.
+Only one recipient is supported.  "))
+                       ((string= org-journal-encryption-extension "age")
+                        (require 'age)
+                        (age-select-keys (age-make-context 'Age age-armor)
+                                         "Select new recipients for encryption."))
+                       (t
+                        (error "Encryption extension \"%s\" is unsupported"
+                               org-journal-encryption-extension)))))
 
   (unless recipient
     (user-error "You need to specify exactly one recipient"))
@@ -1884,29 +1964,36 @@ Only one recipient is supported.  ")))
     (user-error "org-journal encryption not enabled"))
 
   (cl-loop
-   with buf
-   with kill-buffer
-   for journal in (org-journal--list-files)
-   do
-   (setq buf (find-buffer-visiting journal)
-         kill-buffer nil)
+        with buf
+        with kill-buffer
+        for journal-file-path in (org-journal--list-files)
+        if (string-match-p (format "\.%s$" org-journal-encryption-extension) journal-file-path)
+        do
+        (setq buf (find-buffer-visiting journal-file-path)
+              kill-buffer nil)
 
-   (when (and buf
-              (buffer-modified-p buf)
-              (y-or-n-p (format "Journal \"%s\" modified, save before re-encryption?"
-                                (file-name-nondirectory journal))))
-     (save-buffer buf))
+        (when (and buf
+                   (buffer-modified-p buf)
+                   (y-or-n-p (format "Journal \"%s\" modified, save before re-encryption?"
+                                     (file-name-nondirectory journal-file-path))))
+          (save-buffer buf))
 
-   (unless buf
-     (setq kill-buffer t
-           buf (find-file-noselect journal)))
+        (unless buf
+          (setq kill-buffer t
+                buf (find-file-noselect journal-file-path)))
 
-   (with-current-buffer buf
-     (let ((epa-file-encrypt-to (epg-sub-key-id (car (epg-key-sub-key-list (car recipient))))))
-       (set-buffer-modified-p t)
-       (save-buffer)
-       (when kill-buffer
-         (kill-buffer))))))
+        (with-current-buffer buf
+          (let ((epa-file-encrypt-to)
+                (age-file-encrypt-to))
+            (cond
+              ((string= org-journal-encryption-extension "gpg")
+               (setq-local epa-file-encrypt-to (epg-sub-key-id (car (epg-key-sub-key-list (car recipient))))))
+              ((string= org-journal-encryption-extension "age")
+               (setq-local age-file-encrypt-to recipient)))
+            (set-buffer-modified-p t)
+            (save-buffer)
+            (when kill-buffer
+              (kill-buffer))))))
 
 (defun org-journal--decrypt ()
   "Decrypt journal entry at point."
@@ -1929,12 +2016,6 @@ enabling encryption by default."
             'org-journal-encryption-hook
             nil t))
 
-(defcustom org-journal-mode-hook
-  '(turn-on-visual-line-mode
-    org-journal-default-enable-encryption)
-  "Hook to run when org-journal-mode is loaded."
-  :type 'hook)
-
 (provide 'org-journal)
 
 ;;; org-journal.el ends here
